/***************************************************************************
 *   Copyright (C) 2010 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QDebug>
#include <QScrollBar>
#include <QTimer>
#include <math.h>
#include <QFileDialog>
#include <QDesktopWidget>
#include <QLineEdit>
#include <QMessageBox>
#include <QSettings>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    changed = false;
    searchId = "";
    connect(ui->listWidget->verticalScrollBar(),SIGNAL(valueChanged(int)),this,SLOT(updateList()));
    connect(ui->listWidget->verticalScrollBar(),SIGNAL(rangeChanged(int,int)),this,SLOT(updateList()));
    ui->stackedWidget->setCurrentIndex(0);

    //@memo フルスクリーンモードでのQAction有効化のための工夫。
    //      QToolBarをフルスクリーンになるwidgetに追加し、可視化。
    //      hide()してはいけない。高さを0にしてhide()に見せる。
    //      その上でQActionを割り当てればそのアクションはフルスクリーン窓でも実行される。

    minimumRating = new NopViewRatingWidget(ui->ratingBox);
    ui->minimumLayout->addWidget(minimumRating);
    minimumRating->setRating(0);
    maximumRating = new NopViewRatingWidget(ui->ratingBox);
    ui->maximumLayout->addWidget(maximumRating);
    maximumRating->setRating(10);
    connect(minimumRating,SIGNAL(valueChanged(int)),this,SLOT(ratingMinimumValueChanged(int)));
    connect(maximumRating,SIGNAL(valueChanged(int)),this,SLOT(ratingMaximumValueChanged(int)));
    ratingEdit = new NopViewRatingWidget(ui->ratingEditBox);
    ui->ratingEditBox->layout()->addWidget(ratingEdit);
    ratingEdit->setRating(0);
    connect(ratingEdit,SIGNAL(valueChanged(int)),this,SLOT(editRatingValueChanged(int)));


    viewToolBar = new QToolBar(ui->singlePage);
    ui->singlePage->layout()->addWidget(viewToolBar);
    viewToolBar->addAction(ui->actionNext);
    viewToolBar->addAction(ui->actionPrev);
    viewToolBar->addAction(ui->actionFullscreen);
    viewToolBar->addAction(ui->actionFitMode);
    viewToolBar->addAction(ui->actionQuit);
    viewToolBar->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
    viewToolBar->setFixedHeight(0);
    glWidget = new NopViewGLWidget(ui->singlePage);
    ui->singlePage->layout()->addWidget(glWidget);
    glWidget->setFocusPolicy(Qt::NoFocus);

    catalog = new NopViewCatalog(this);
    connect(catalog,SIGNAL(allTags(QStringList)),this,SLOT(allTagsUpdate(QStringList)));
    connect(catalog,SIGNAL(result(QString,QString)),this,SLOT(resultReceiver(QString,QString)));
    connect(catalog,SIGNAL(searchFinished(QString)),this,SLOT(searchFinished(QString)));
    connect(catalog,SIGNAL(nameComboList(QStringList)),this,SLOT(nameComboList(QStringList)));
    connect(catalog,SIGNAL(fileInfo(QString,QStringList,int)),this,SLOT(fileInfoReceiver(QString,QStringList,int)));
    connect(catalog,SIGNAL(changed()),this,SLOT(dataChanged()));

    //catalog->loadCatalog("./nopview.dat");

    //catalog->search(searchId);

    connect(ui->taggedListWidget,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(search()));
    connect(ui->notTaggedListWidget,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(search()));
    connect(ui->editDockAddTagCombo->lineEdit(),SIGNAL(returnPressed()),this,SLOT(on_editDockAddTagPushButton_clicked()));
    show();
    loadSetting();
}

MainWindow::~MainWindow()
{
    saveSetting();
    delete ui;
}

void MainWindow::allTagsUpdate(QStringList tags)
{
    // NopViewCatalogからタグリストを受け取った時の動作。
    QStringList tagged,nottagged,edittaged;
    QListIterator<QListWidgetItem *> tit(ui->taggedListWidget->selectedItems());
    while(tit.hasNext()) tagged.append(tit.next()->text());
    QListIterator<QListWidgetItem *> nit(ui->notTaggedListWidget->selectedItems());
    while(nit.hasNext()) nottagged.append(nit.next()->text());
    QListIterator<QListWidgetItem *> eit(ui->editDockTagListWidget->selectedItems());
    while(eit.hasNext()) edittaged.append(eit.next()->text());
    ui->taggedListWidget->clear();
    ui->notTaggedListWidget->clear();
    ui->editDockTagListWidget->clear();
    QStringListIterator it(tags);
    while(it.hasNext()) {
        QString t = it.next();
        QListWidgetItem *item = new QListWidgetItem(ui->taggedListWidget);
        item->setText(t);
        if (tagged.contains(t)) item->setSelected(true);
        item = new QListWidgetItem(ui->notTaggedListWidget);
        item->setText(t);
        if (nottagged.contains(t)) item->setSelected(true);
        item = new QListWidgetItem(ui->editDockTagListWidget);
        item->setText(t);
        if (edittaged.contains(t)) item->setSelected(true);
    }
    // editDockのタグ追加用コンボにも突っ込むぜ。
    ui->editDockAddTagCombo->clear();
    ui->editDockAddTagCombo->addItems(tags);
    ui->editDockAddTagCombo->lineEdit()->clear();
    //@memo on_listWidget_itemSelectionChanged()を呼ぶことでeditTagListWidgetを更新する。
    on_listWidget_itemSelectionChanged();
}

void MainWindow::resultReceiver(const QString id, const QString filename)
{
    // 検索の結果を受け取る。
    if (id != searchId) return;
    QListWidgetItem *item = new QListWidgetItem(ui->listWidget);
    item->setText(filename);
    item->setSizeHint(QSize(ui->listWidget->gridSize().width(),ui->listWidget->gridSize().height()));
    item->setTextAlignment(Qt::AlignHCenter | Qt::AlignBottom);

    if (filename == currentFilename) {
        ui->listWidget->setCurrentItem(item);
    }
}

void MainWindow::search()
{
    // 現在のfilterDockの状態から検索をかける。
    if (ui->listWidget->currentItem()) currentFilename = ui->listWidget->currentItem()->text();
    else currentFilename = "";
    thumbQueue.clear();
    thumbed.clear();
    ui->listWidget->clear();
    QListIterator<QListWidgetItem *> tit(ui->taggedListWidget->selectedItems());
    QStringList tagged;
    while(tit.hasNext()) tagged.append(tit.next()->text());
    QListIterator<QListWidgetItem *> nit(ui->notTaggedListWidget->selectedItems());
    QStringList nottagged;
    while(nit.hasNext()) nottagged.append(nit.next()->text());
    QString namesearch;
    if (ui->nameComboCheck->isChecked()) namesearch = ui->nameCombo->lineEdit()->text();
    catalog->search(searchId,tagged,nottagged,
                    minimumRating->rating(),maximumRating->rating(),
                    looklike,namesearch);
}

void MainWindow::updateList()
{
    // リストのスクロール位置から表示すべきアイテムを割り出し、
    // アイコンの作成をしかける変態コード。
    while(thumbed.count() > 300) {
        if (!thumbed.count()) break;
            thumbed.first()->setIcon(QIcon());
            thumbed.removeFirst();
    }
    thumbQueue.clear();
    QScrollBar *slider = ui->listWidget->verticalScrollBar();
    int pos = slider->value();
    int col = (ui->listWidget->viewport()->width() / ui->listWidget->gridSize().width());
    if (col <= 0) col = 1;
    int row = (ui->listWidget->viewport()->height() / ui->listWidget->gridSize().height()) + 1;
    int offset = 0;
    if (slider->maximum() && pos) {
        offset = floor(((float)(ui->listWidget->count() - (col * row)) / (float)slider->maximum()) * (float)pos);
    }
    for(int i =  offset - (col * 1);i < offset + (col * (row + 1));i ++) {
        if (i < 0) continue;
        if (i >= ui->listWidget->count()) break;
        if (ui->listWidget->item(i)->icon().isNull())
            thumbQueue.append(ui->listWidget->item(i));
    }
    QTimer::singleShot(500,this,SLOT(thumbnailProc()));
}

void MainWindow::thumbnailProc()
{
    // アイテムのサムネイルを実際に処理するコード。
    // シングルスレッドでレスポンスを上げるため、タイマーを利用してディレイ。
    // 疑似イベントドリブンマルチタスク。またの名をうんこ。
    // あとメモリを食い尽くさないようにアイコンをキャッシュしてみたりする変態コード。
    if (thumbQueue.count()) {
        if (!iconCache.contains(thumbQueue[0]->text())) {
            QImage p(thumbQueue[0]->text());
            QImage icon = p.scaled(300,300,Qt::KeepAspectRatio,Qt::SmoothTransformation);
            iconCache[thumbQueue[0]->text()] = icon;
            iconCacheAges.append(thumbQueue[0]->text());
            if (iconCache.count() > 500) {
                iconCache.remove(iconCacheAges.first());
                iconCacheAges.removeFirst();
            }
        }
        if (thumbQueue[0]->icon().isNull()) {
            thumbQueue[0]->setIcon(QPixmap::fromImage(iconCache[thumbQueue[0]->text()]));
        }
        if (thumbed.contains(thumbQueue[0])) thumbed.removeOne(thumbQueue[0]);
        thumbed.append(thumbQueue[0]);
        thumbQueue.removeFirst();
        QTimer::singleShot(50,this,SLOT(thumbnailProc()));
    }
    return;
}

void MainWindow::searchFinished(const QString id)
{
    // 「検索終わりましたー！」
    if (id != searchId) return;
    QTimer::singleShot(500,this,SLOT(ensureVisible()));
    updateList();
}

void MainWindow::ensureVisible()
{
    // 今いるアイテムを見えるようにスクロールする。
    if (ui->listWidget->currentItem()) ui->listWidget->scrollTo(ui->listWidget->currentIndex());
}

void MainWindow::nameComboList(const QStringList combo)
{
    // ファイル名検索のコンボリストはNopCatalogが作ってそれを貰います。
    ui->nameCombo->addItems(combo);
}

void MainWindow::fileInfoReceiver(const QString filename, const QStringList tags, int rating)
{
    // ファイル情報検索のデータを受け取る。
    // 現在選択されているファイルの数で動作を変えます。
    if (ui->listWidget->selectedItems().count() <= 1) {
        ui->editDockFileName->setText(filename);
        ratingEdit->setRating(rating);
        ui->editDockTagListWidget->clearSelection();
        if (iconCache.contains(filename)) ui->editDockFilePreview->setPixmap(
                    QPixmap::fromImage(iconCache[filename]).scaled(200,200,Qt::KeepAspectRatio,Qt::SmoothTransformation));
        else {
            QIcon icon(filename);
            ui->editDockFilePreview->setPixmap(icon.pixmap(200,200));
        }
    } else {
        if (rating < ratingEdit->rating()) ratingEdit->setRating(rating);
    }
    for(int i = 0;i < ui->editDockTagListWidget->count();i ++) {
        if (tags.contains(ui->editDockTagListWidget->item(i)->text()))
                ui->editDockTagListWidget->item(i)->setSelected(true);
    }
}

void MainWindow::ratingMinimumValueChanged(int value)
{
    // 最小Ratingが変更された。
    if (maximumRating->rating() < value) maximumRating->setRating(value);
    search();
}

void MainWindow::ratingMaximumValueChanged(int value)
{
    // 最大Ratingが変更された。
    if (minimumRating->rating() > value) minimumRating->setRating(value);
    search();
}

void MainWindow::editRatingValueChanged(int rating)
{
    // Ratingが変更された。
    QListIterator<QListWidgetItem *> it(ui->listWidget->selectedItems());
    while(it.hasNext()) {
        catalog->setRating(it.next()->text(),rating);
    }
}

void MainWindow::dataChanged()
{
    changed = true;
}

void MainWindow::loadSetting()
{
    QSettings setting("nop.net-p.org","nopView");
    resize(setting.value("WindowSize").toSize());
    if (setting.value("WindowMaximized").toBool()) showMaximized();
//    ui->filterDock->resize(setting.value("FilterDockSize").toSize());
//    ui->editDock->resize(setting.value("EditDockSize").toSize());

    if (!setting.value("FilterDockVisible").toBool()) ui->actionShowFilterDock->trigger();
    if (!setting.value("EditDockVisible").toBool()) ui->actionShowEditDock->trigger();
    if (setting.value("Fitmode").toBool()) ui->actionFitMode->trigger();
    ui->zoomSlider->setValue(setting.value("IconSize").toInt());
}

void MainWindow::saveSetting()
{
    QSettings setting("nop.net-p.org","nopView");
    setting.setValue("WindowSize",size());
    setting.setValue("WindowMaximized",isMaximized());
//    setting.setValue("FilterDockSize",ui->filterDock->size());
//    setting.setValue("EditDockSize",ui->editDock->size());

    setting.setValue("FilterDockVisible",ui->actionShowFilterDock->isChecked());
    setting.setValue("EditDockVisible",ui->actionShowEditDock->isChecked());
    setting.setValue("Fitmode",ui->actionFitMode->isChecked());
    setting.setValue("IconSize",ui->zoomSlider->value());

}

void MainWindow::close()
{
    if (changed) {
        // 「いいのかい？ホイホイ終了しちまって？」
        if (QMessageBox::question(this,tr("warning."),
            tr("Catalog was changed. Do you close?"),
            QMessageBox::Ok,QMessageBox::Cancel) == QMessageBox::Cancel) {
            return;
        }

    }
    QMainWindow::close();
}

//
// from Designer ----
//


void MainWindow::on_zoomSlider_valueChanged(int value)
{
    // ズームスライダがいぢられた。
    //@memo QApplication::desktop()->screen()->logicalDpiY() / 72.0
    //      1ポイントをピクセルに換算する式。floatにキャストして使うと良い。
    ui->listWidget->setGridSize(QSize(value,value));
    ui->listWidget->setIconSize(QSize(value,value -
                                      ((float)QApplication::desktop()->screen()->logicalDpiY() / 72.0) *
                                      ui->listWidget->font().pointSize()));
    for(int i = 0;i < ui->listWidget->count();i ++) {
        ui->listWidget->item(i)->setSizeHint(QSize(ui->listWidget->gridSize().width(),
                                                   ui->listWidget->gridSize().height()));
    }
    QTimer::singleShot(500,this,SLOT(ensureVisible()));
}

void MainWindow::on_actionSingleView_triggered(bool checked)
{
    // シングルビューに切り替えた。
    if (checked) {
        thumbQueue.clear();
        ui->stackedWidget->setCurrentIndex(1);
        if (ui->listWidget->currentItem()) {
            glWidget->loadFile(ui->listWidget->currentItem()->text());
        }
    } else {
        ui->stackedWidget->setCurrentIndex(0);
        QTimer::singleShot(500,this,SLOT(ensureVisible()));
        updateList();
    }
}

void MainWindow::on_listWidget_currentItemChanged(QListWidgetItem* current, QListWidgetItem* previous)
{
    // カレントアイテムが変更された。
    if (previous) {
        // 直前選択エントリのデータ更新処理
    }

    if (current) {
        if (ui->stackedWidget->currentIndex() == 1) {
            glWidget->loadFile(current->text());
        }
    }
}

void MainWindow::on_actionNext_triggered()
{
    // 次のアイテムへ。
    if (ui->listWidget->currentRow() >= ui->listWidget->count() - 1) return;
    ui->listWidget->clearSelection();
    ui->listWidget->setCurrentRow(ui->listWidget->currentRow() + 1);
}

void MainWindow::on_actionPrev_triggered()
{
    // 前のアイテムへ。
    if (ui->listWidget->currentRow() <= 0) return;
    ui->listWidget->clearSelection();
    ui->listWidget->setCurrentRow(ui->listWidget->currentRow() - 1);
}

void MainWindow::on_actionFullscreen_triggered(bool checked)
{
    // フルスクリーンモード。
    if (checked) {
        pSingle = ui->actionSingleView->isChecked();
        if (!pSingle) ui->actionSingleView->activate(QAction::Trigger);
        pMaximized = isMaximized();
        ui->mainToolBar->hide();
        ui->menuBar->hide();
        ui->statusBar->hide();
        ui->filterDock->hide();
        ui->editDock->hide();
        showFullScreen();
    } else {
        //@memo showMaxmized()はshowFullscreenから直接はうまくいかない(X)。
        ui->mainToolBar->show();
        ui->menuBar->show();
        ui->statusBar->show();
        if (ui->actionShowEditDock->isChecked()) ui->editDock->show();
        if (ui->actionShowFilterDock->isChecked()) ui->filterDock->show();
        showNormal();
        if (!pSingle) ui->actionSingleView->activate(QAction::Trigger);
        if (pMaximized) showMaximized();
    }
}

void MainWindow::on_actionOpen_triggered()
{
    // カタログを読み込む。
    QString f = QFileDialog::getOpenFileName(0,tr("Open Catalog."));
    if (f.isNull()) return;
    catalog->loadCatalog(f);
    changed = false;
    search();
}

void MainWindow::on_actionFitMode_triggered(bool checked)
{
    // singleViewのフィットモード。
    glWidget->setFitMode(checked);
}

void MainWindow::on_actionSaveAs_triggered()
{
    // 名前を付けて保存。
    QString f = QFileDialog::getOpenFileName(0,tr("Save Catalog."));
    if (!catalog->saveCatalog(f)) setStatusTip(tr("save failed."));
    else changed = false;
}

void MainWindow::on_actionLooklike_triggered(bool checked)
{
    // 類似画像検索。
    if (!checked) {
        looklike = "";
        search();
        return;
    }
    if (ui->listWidget->currentItem())
    looklike = ui->listWidget->currentItem()->text();
    search();
}

void MainWindow::on_nameCombo_editTextChanged(QString)
{
    // 名前コンボが編集された。
    if (ui->nameComboCheck->isChecked()) search();
}

void MainWindow::on_nameComboCheck_clicked()
{
    // 名前コンボ有効チェックが押された。
    search();
}

void MainWindow::on_ratingResetButton_clicked()
{
    // Ratingをリセットするボタンが押された。
    maximumRating->setRating(10);
    minimumRating->setRating(0);
    search();
}

void MainWindow::on_actionAboutNopView_triggered()
{
    // aboutです。
    QMessageBox::about(this,tr("About nopView"),
                       QString("nopView\n$Rev: 21 $\nAuthor: TAM(tam-t@par.odn.ne.jp)\n(C)2011"));
}

void MainWindow::on_editDockAddTagPushButton_clicked()
{
    // タグ追加ボタンが押された。
    QListIterator<QListWidgetItem *> it(ui->listWidget->selectedItems());
    QString tag = ui->editDockAddTagCombo->lineEdit()->text();
    while(it.hasNext()) {
        catalog->addTag(it.next()->text(),tag);
    }
}

void MainWindow::on_editDockTagListWidget_itemClicked(QListWidgetItem* item)
{
    // タグ編集リストが変更された。
    //@memo itemSelectionChangedでは読み込みでタグを更新してもこれが呼ばれてしまう。
    //      itemClickedで対処。
    QListIterator<QListWidgetItem *> it(ui->listWidget->selectedItems());
    while(it.hasNext()) {
        QListWidgetItem *i = it.next();
        if (item->isSelected()) catalog->addTag(i->text(),item->text());
        else catalog->removeTag(i->text(),item->text());
    }
}

void MainWindow::on_actionAddFile_triggered()
{
    // ファイル追加。
    QStringList files = QFileDialog::getOpenFileNames(this,tr("Add Files"));
    QStringListIterator it(files);
    while(it.hasNext()) catalog->addFile(it.next());
    search();
}

void MainWindow::on_listWidget_itemSelectionChanged()
{
    // ファイルリストの選択が変更された。
    if (!ui->listWidget->selectedItems().count()) {
        ui->editDockFilePreview->setPixmap(QPixmap());
        ui->editDockFilePreview->setText(tr("."));
        ui->editDockTagListWidget->clearSelection();
        ui->editDockFileName->setText(QString());
        return;
    }
    if (ui->listWidget->selectedItems().count() == 1) {
        catalog->getFileInfo(ui->listWidget->selectedItems().at(0)->text());
        return;
    }
    ui->editDockFilePreview->setPixmap(QPixmap());
    ui->editDockFilePreview->setText(tr("%1 files selected.").arg(ui->listWidget->selectedItems().count()));
    ui->editDockTagListWidget->clearSelection();
    ui->editDockFileName->setText(QString());
    QListIterator<QListWidgetItem *> it(ui->listWidget->selectedItems());
    while(it.hasNext()) catalog->getFileInfo(it.next()->text());
}

void MainWindow::on_editDockRemoveButton_clicked()
{
    // 選択ファイルをカタログから除去するボタン。
    if (!ui->listWidget->selectedItems().count()) return;
    if (QMessageBox::question(this,tr("File remove from catalog."),
            QString::number(ui->listWidget->selectedItems().count()) +
                              tr(" file(s) are removed now. ok?"),
                              QMessageBox::Ok,QMessageBox::Cancel) == QMessageBox::Ok) {
        QListIterator<QListWidgetItem *> it(ui->listWidget->selectedItems());
        while(it.hasNext()) {
            QListWidgetItem *i = it.next();
            catalog->removeFile(i->text());
        }
        search();
    }
}

void MainWindow::on_taggedAllButton_clicked()
{
    ui->taggedListWidget->selectAll();
    search();
}

void MainWindow::on_taggedResetButton_clicked()
{
    ui->taggedListWidget->clearSelection();
    search();
}

void MainWindow::on_notTaggedAllButton_clicked()
{
    ui->notTaggedListWidget->selectAll();
    search();
}

void MainWindow::on_notTaggedResetButton_clicked()
{
    ui->notTaggedListWidget->clearSelection();
    search();
}

void MainWindow::on_actionSave_triggered()
{
    if (!catalog->saveCatalog()) return on_actionSaveAs_triggered();
    changed = false;
    return;
}
