/***************************************************************************
 *   Copyright (C) 2010 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QMap>
#include <QIcon>
#include <QToolBar>
#include <QListWidgetItem>

#include "nopviewcatalog.h"
#include "nopviewglwidget.h"
#include "nopviewratingwidget.h"

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

    NopViewCatalog *catalog;
private:
    QString searchId;
    Ui::MainWindow *ui;
    QMap<QString,QImage> iconCache;
    QStringList iconCacheAges;
    QList<QListWidgetItem *> thumbed;
    QList<QListWidgetItem *> thumbQueue;
    NopViewGLWidget *glWidget;
    QToolBar *viewToolBar;
    QString currentFilename;
    QString looklike;
    bool changed;
    bool pMaximized;
    bool pSingle;
    NopViewRatingWidget *minimumRating;
    NopViewRatingWidget *maximumRating;
    NopViewRatingWidget *ratingEdit;
public slots:
    void allTagsUpdate(QStringList tags);
    void resultReceiver(const QString id,const QString filename);
    void search();
    void updateList();
    void thumbnailProc();
    void searchFinished(const QString id);
    void ensureVisible();
    void nameComboList(const QStringList combo);
    void fileInfoReceiver(const QString filename,const QStringList tags,int rating);
    void ratingMinimumValueChanged(int value);
    void ratingMaximumValueChanged(int value);
    void editRatingValueChanged(int rating);
    void loadSetting();
    void saveSetting();
    void dataChanged();
    void close();

private slots:
    void on_zoomSlider_valueChanged(int value);

    void on_actionSingleView_triggered(bool checked);

    void on_listWidget_currentItemChanged(QListWidgetItem* current, QListWidgetItem* previous);

    void on_actionNext_triggered();

    void on_actionPrev_triggered();

    void on_actionFullscreen_triggered(bool checked);

    void on_actionOpen_triggered();

    void on_actionFitMode_triggered(bool checked);

    void on_actionSaveAs_triggered();

    void on_actionLooklike_triggered(bool checked);

    void on_nameCombo_editTextChanged(QString);

    void on_nameComboCheck_clicked();

    void on_ratingResetButton_clicked();

    void on_actionAboutNopView_triggered();

    void on_editDockAddTagPushButton_clicked();

    void on_editDockTagListWidget_itemClicked(QListWidgetItem* item);

    void on_actionAddFile_triggered();

    void on_listWidget_itemSelectionChanged();

    void on_editDockRemoveButton_clicked();

    void on_taggedAllButton_clicked();

    void on_taggedResetButton_clicked();

    void on_notTaggedAllButton_clicked();

    void on_notTaggedResetButton_clicked();

    void on_actionSave_triggered();

signals:
};

#endif // MAINWINDOW_H
