/***************************************************************************
 *   Copyright (C) 2011 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "nopviewcatalog.h"
#include <QDebug>
#include <QFileDialog>
#include <QImage>

NopViewCatalog::NopViewCatalog(QObject *parent) :
    QThread(parent)
{
    catalogName = "";
}

NopViewCatalog::~NopViewCatalog()
{
}

void NopViewCatalog::run()
{
    exec();
}

void NopViewCatalog::loadCatalog(const QString catalog)
{
    data.clear();

    QFile file(catalog);
    if (file.open(QIODevice::ReadOnly) == false) return;
    QDataStream in(&file);

    QString header;
    in >> header;
    if (header == "nopViewCatalog v1") {
        loadCatalog_v1(in);
    }
    catalogName = catalog;
    file.close();
}

bool NopViewCatalog::saveCatalog()
{
    if (catalogName.isEmpty()) return false;
    return saveCatalog(catalogName);
}

bool NopViewCatalog::saveCatalog(const QString catalog)
{
    QFile file(catalog);
    if (file.open(QIODevice::WriteOnly) == false) return false;
    QDataStream out(&file);

    saveCatalog_v1(out);
    file.close();
    return true;

}

void NopViewCatalog::search(const QString id, const QString filename)
{
    if (data.contains(filename)) emit result(id,data[filename].filename);
}

void NopViewCatalog::search(const QString id,
                            const QStringList tagged, const QStringList nottagged,
                            int rateingminimum, int rateingmaximum,
                            const QString looklike,const QString name)
{
    QList<QRgb> fingerPrint;
    if (data.contains(looklike)) fingerPrint = data[looklike].fingerprint;
    QMapIterator<QString,NopViewCatalogEntry> it(data);
    while(it.hasNext()) {
        NopViewCatalogEntry e = it.next().value();
        if (!name.isEmpty()) {
            if (e.filename.left(name.count()) != name) continue;
        }
        if (!fingerPrint.isEmpty()) {
            // 類似画像検索。15一致でゴー。
            int sc = 0;
            for(int i = 0;i < 25;i ++)
                if (fingerPrint.at(i) == e.fingerprint.at(i)) sc ++;
            if (sc < 15) continue;
        }
        if ((e.rating < rateingminimum) || (e.rating > rateingmaximum)) continue;
        bool check = true;
        QStringListIterator nt(nottagged);
        while(nt.hasNext()) if (e.tagged.contains(nt.next())) {
            check = false;
            break;
        }
        if (!check) continue;
        check = true;
        QStringListIterator tg(tagged);
        while(tg.hasNext()) if (!e.tagged.contains(tg.next())) {
            check = false;
            break;
        }
        if (!check) continue;
        emit result(id,e.filename);
    }
    emit searchFinished(id);

}

void NopViewCatalog::stopSearch(const QString id)
{

}

void NopViewCatalog::createFingerprint(const QString filename)
{
    qDebug() << filename;
    if (!data.contains(filename)) return;
    QImage im(filename);
    if (im.isNull()) return;
    data[filename].fingerprint.clear();
    QImage s = im.scaled(QSize(5,5));
    QImage sc = s.convertToFormat(QImage::Format_RGB444);
    for(int i = 0;i < 5;i ++) {
        for(int j = 0;j < 5;j ++) {
            data[filename].fingerprint.append(sc.pixel(j,i));
        }
    }
}

void NopViewCatalog::saveCatalog_v1(QDataStream &out)
{
    out << QString("nopViewCatalog v1");
    QMapIterator<QString,NopViewCatalogEntry> it(data);
    while(it.hasNext()) {
        NopViewCatalogEntry e = it.next().value();
        out << e.filename;
        out << e.tagged;
        out << e.rating;
        out << e.managed;
        out << e.fingerprint;
    }
}

void NopViewCatalog::loadCatalog_v1(QDataStream &in)
{
    tags.clear();
    while(!in.atEnd()) {
        NopViewCatalogEntry e;
        in >> e.filename;
        in >> e.tagged;
        in >> e.rating;
        in >> e.managed;
        in >> e.fingerprint;
        QStringListIterator it(e.tagged);
        while(it.hasNext()) tags.insert(it.next());
        data[e.filename] = e;
        QFileInfo f(e.filename);
        if (!dirCombo.contains(f.absolutePath())) dirCombo.append(f.absolutePath());
        if (!e.fingerprint.count()) createFingerprint(e.filename);
//        qDebug() << data[e.filename].fingerprint;
    }
    emit allTags(tags.toList());
    emit nameComboList(dirCombo);
}

void NopViewCatalog::addFile(const QString filename)
{
    if (data.contains(filename)) return;
    NopViewCatalogEntry e;
    e.filename = filename;
    e.rating = 0;
    data[filename] = e;
    QFileInfo f(e.filename);
    createFingerprint(filename);
    if (!dirCombo.contains(f.absolutePath())) {
        dirCombo.append(f.absolutePath());
        emit nameComboList(dirCombo);
    }
    emit changed();
}

void NopViewCatalog::removeFile(const QString filename)
{
    if (data.contains(filename)) {
        data.remove(filename);
        emit changed();
    }
}

void NopViewCatalog::getFileInfo(const QString filename)
{
    if (!data.contains(filename)) return;
    emit fileInfo(data[filename].filename,data[filename].tagged,data[filename].rating);
}

void NopViewCatalog::setEntryData(const QString filename, const QStringList tagged, int rating)
{
    if (!data.contains(filename)) return;
    data[filename].tagged = tagged;
    data[filename].rating = rating;
    QStringListIterator it(tagged);
    while(it.hasNext()) {
        QString t = it.next();
        if (!tags.contains(t)) tags.insert(t);
    }
    emit allTags(tags.toList());
    emit changed();
}
void NopViewCatalog::addTag(const QString filename, const QString tag)
{
    if (tag.isEmpty()) return;
    if (!data.contains(filename)) return;
    if (data[filename].tagged.contains(tag)) return;
    data[filename].tagged.append(tag);
    if (!tags.contains(tag)) {
        tags.insert(tag);
        emit allTags(tags.toList());
        emit changed();
    }
}

void NopViewCatalog::removeTag(const QString filename, const QString tag)
{
    if (!data.contains(filename)) return;
    if (!data[filename].tagged.contains(tag)) return;
    data[filename].tagged.removeAll(tag);
    emit changed();
}

void NopViewCatalog::setRating(const QString filename, int rating)
{
    if (!data.contains(filename)) return;
    data[filename].rating = rating;
    emit changed();
}

