/***************************************************************************
 *   Copyright (C) 2011 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef NOPVIEWCATALOG_H
#define NOPVIEWCATALOG_H

#include <QThread>
#include <QStringList>
#include <QList>
#include <QMap>
#include <QRgb>
#include <QSet>

class NopViewCatalogEntry
{
public:
    QString filename;
    QStringList tagged;
    int rating;
    bool managed;
    QList<QRgb> fingerprint;
};

class NopViewCatalog : public QThread
{
    Q_OBJECT
public:
    explicit NopViewCatalog(QObject *parent = 0);
    ~NopViewCatalog();
    void run();
private:
    void loadCatalog_v1(QDataStream &in);
    void saveCatalog_v1(QDataStream &out);
private:
    QMap<QString,NopViewCatalogEntry> data;
    QSet<QString> tags;
    QStringList dirCombo;
    QString catalogName;
signals:
    void result(const QString id,const QString file);
    void allTags(const QStringList tags);
    void searchFinished(const QString id);
    void nameComboList(const QStringList list);
    void fileInfo(const QString filename,const QStringList tagged,int rating);
    void changed();
public slots:
    void loadCatalog(const QString catalog);
    bool saveCatalog(const QString catalog);
    bool saveCatalog();
    void createFingerprint(const QString filename);
    void search(const QString id,
                const QStringList tagged = QStringList(),const QStringList nottagged = QStringList(),
                int rateingminimum = 0,int rateingmaximum = 10,
                const QString looklike = QString(),
                const QString name = QString());
    void search(const QString id,const QString filename);
    void stopSearch(const QString id);
    void getFileInfo(const QString filename);
    void setEntryData(const QString filename,const QStringList tagged,int rating);
    void addTag(const QString filename,const QString tag);
    void removeTag(const QString filename,const QString tag);
    void setRating(const QString filename,int rating);
    void addFile(const QString filename);
    void removeFile(const QString filename);
};

#endif // NOPVIEWCATALOG_H
