/***************************************************************************
 *   Copyright (C) 2011 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "nopviewglwidget.h"
#include <QDebug>

NopViewGLWidget::NopViewGLWidget(QWidget *parent) :
    QGLWidget(parent)
{
    angle = 0;
    posX = 0;
    posY = 0;
    zoom = 1;
    fit = false;
    lbutton = false;
    rbutton = false;
    ppoint = QPoint(0,0);
    initializeGL();
}

NopViewGLWidget::~NopViewGLWidget()
{
}

void NopViewGLWidget::initializeGL()
{
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glShadeModel(GL_SMOOTH);
    //@memo 以下mingw環境でコンパイルできないので外す。ってかｲﾗﾈ。
    //glEnable(GL_MULTISAMPLE);
}

void NopViewGLWidget::paintGL()
{
    // 縦横の座標を１ピクセル＝1.0に合わせる。
    // 落ち着いて素数を数えれば分かる。
    double wl = 0 - ((double)width() / 2);
    double wh = (double)width() / 2;
    double hl = 0 - ((double)height() / 2);
    double hh = (double)height() / 2;
    double twl = 0 - ((double)tex.width() / 2);
    double twh = (double)tex.width() / 2;
    double thl = 0 - ((double)tex.height() / 2);
    double thh = (double)tex.height() / 2;
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(wl, wh, hl, hh, -1.0, 1.0);
    glPushMatrix();

    if (zoom < 0) zoom = 0;

    if (fit) {
        // フィットモード。
        // 画面に収まる拡大率は？。
        double fitzoom;
        if (((double)width() / (double)height()) < ((double)tex.width() / (double)tex.height())) {
            fitzoom = (double)width() / (double)tex.width();
        } else {
            fitzoom = (double)height() / (double)tex.height();
        }
        glScaled(fitzoom,fitzoom,0);
        glTranslated(0,0,0);
        glRotated(0,0,0,1);
    } else {
        // フリーモード。
        //@memo この順番大事。scaled→translated→rotated。
        glScaled(zoom,zoom,0);
        glTranslated((double)posX,(double)posY,0);
        glRotated(angle,0,0,1);
    }

    if (!tex.isNull()) {
        // ﾃｸﾏｸﾏﾔｺﾝﾃｸﾏｸﾏﾔｺﾝ
        glBegin(GL_QUADS);
        glTexCoord2d(0.0,0.0);glVertex2d(twl,thl);
        glTexCoord2d(1.0,0.0);glVertex2d(twh,thl);
        glTexCoord2d(1.0,1.0);glVertex2d(twh,thh);
        glTexCoord2d(0.0,1.0);glVertex2d(twl,thh);
        glEnd();
    }
    glPopMatrix();
    glFlush();

}

void NopViewGLWidget::resizeGL(int width, int height)
{
    glViewport(0, 0, width, height);
    updateGL();
}

void NopViewGLWidget::loadFile(const QString filename)
{
    QImage img(filename);
    if (img.isNull()) return;
    angle = 0;
    posX = 0;
    posY = 0;
    zoom = 1;
    lbutton = false;
    rbutton = false;

    // テクスチャを作って0番に割り当て。
    // 順番大事。
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glEnable(GL_TEXTURE_2D);
    tex = QGLWidget::convertToGLFormat(img);

    glGenTextures(1,&texture);
    glBindTexture(GL_TEXTURE_2D,texture);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

    //@memo glTexImage2DはOpenGL2.0以降でないと画像サイズに制限がかかる。
    //      具体的には2のn乗。有名なやつ。
    //      調整がめんどいならgluBuild2DMipmapsで代用できる。

    //glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA, tex.width(), tex.height(),0, GL_RGBA,GL_UNSIGNED_BYTE, tex.bits());
    gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA, tex.width(), tex.height(), GL_RGBA,GL_UNSIGNED_BYTE, tex.bits());
    updateGL();

}

void NopViewGLWidget::mousePressEvent(QMouseEvent *event)
{
    ppoint = event->pos();
    if (event->button() == Qt::LeftButton) lbutton = true;
    if (event->button() == Qt::RightButton) rbutton = true;
    return;
}

void NopViewGLWidget::mouseReleaseEvent(QMouseEvent *event)
{
    ppoint = event->pos();
    if (event->button() == Qt::LeftButton) lbutton = false;
    if (event->button() == Qt::RightButton) rbutton = false;
}

void NopViewGLWidget::mouseMoveEvent(QMouseEvent *event)
{
    if ((!rbutton) && (!lbutton)) {
        ppoint = event->pos();
        return;
    }
    if (lbutton) {
        posX += (event->x() - ppoint.x());
        posY += (event->y() - ppoint.y()) * -1;
    }
    if (rbutton) {
        angle += (event->x() - ppoint.x());
        zoom += (double)(event->y() - ppoint.y()) / 60.0;
    }
    ppoint = event->pos();
    update();
    return;
}

void NopViewGLWidget::wheelEvent(QWheelEvent *event)
{
    if (rbutton) {
        angle += (double)(event->delta()) / 45;
    } else {
        zoom += 4.0 / (double)(event->delta());
    }
    update();
    return;
}

void NopViewGLWidget::keyPressEvent(QKeyEvent *event)
{
}

void NopViewGLWidget::setFitMode(bool checked)
{
    if (checked) fit = true;
    else {
        fit = false;
        angle = 0;
        posX = 0;
        posY = 0;
        zoom = 1;
    }
    updateGL();
}
