/***************************************************************************
 *   Copyright (C) 2011 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "nopviewratingwidget.h"
#include <QHBoxLayout>
#include <QSpacerItem>
#include <QMouseEvent>

NopViewRatingWidget::NopViewRatingWidget(QWidget *parent) :
    QWidget(parent)
{
    pressed = false;
    _rating = 0;
    setMouseTracking(true);

    QHBoxLayout *lay = new QHBoxLayout(this);
    lay->setSpacing(0);
    lay->setMargin(0);
    lay->addStretch();
    for(int i = 0;i < 5;i ++) {
        QLabel *l = new QLabel(this);
        l->setAlignment(Qt::AlignHCenter | Qt::AlignCenter);
        l->setFixedSize(l->height(),l->height());
        l->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
        l->setFrameStyle(QFrame::StyledPanel | QFrame::Plain);
        lay->addWidget(l);
        stars.append(l);
    }
    lay->addStretch();
    setLayout(lay);
    updateRating();
}

NopViewRatingWidget::~NopViewRatingWidget()
{
}

int NopViewRatingWidget::rating()
{
    return _rating;
}

void NopViewRatingWidget::setRating(int r)
{
    _rating = r;
    updateRating();
}

void NopViewRatingWidget::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) pressed = true;
    setRatingFromPos(event->pos().x());
}

void NopViewRatingWidget::mouseReleaseEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) pressed = false;
}

void NopViewRatingWidget::mouseMoveEvent(QMouseEvent *event)
{
    if (!pressed) return;
    setRatingFromPos(event->pos().x());
}

void NopViewRatingWidget::setRatingFromPos(int x)
{
    x -= stars[0]->x();
    if (x <= 0) {
        _rating = 0;
    } else if (x >= stars[0]->width() * 5) {
        _rating = 10;
    } else for(int i = 0;i < 5;i ++) {
        if ((x > stars[0]->width() * i) && (x < stars[0]->width() * (i + 1))) {
            if (x < (stars[0]->width() * i) + (stars[0]->width() / 2)) {
                _rating = (i * 2) + 1;
            } else {
                _rating = (i * 2) + 2;
            }
            break;
        }
    }
    updateRating();
    emit valueChanged(_rating);
}

void NopViewRatingWidget::updateRating()
{
    for(int i = 0;i < 5;i ++) stars[i]->setText(tr(""));
    for(int i = 0;i < _rating / 2;i ++) {
        stars[i]->setText(tr("*"));
    }
    if ((_rating < 10) && (_rating % 2)) stars[(_rating / 2)]->setText(tr("+"));
}
